	OPTION EXPLICIT
	OPTION DEFAULT NONE
	CPU 48
	GOTO TestSuite

'************************************************************************************* 
' 
' Peter Mather
' 
' 3.0 
' 2015-28-6 
' 
' Graphics library with drivers for S1289 display 
' 16-bit parallel output from Micromite MX170 - flexible pinout
'
' Touch Input Library with drivers for XPT2046
'
' drawing primitives assume a graphics display where top left is point 0, 0
' function calls are as compatible as possible with the in-built graphics routines as documented for version 4.7
'
' Define which pins are used to interface with the display
' Call S1289.PinDefs BEFORE calling anything else in S1289
' Sub S1289.PinDefs(D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15,RS,RD,WR,RST)
'	  D0-D15 Pins used for DB0-DB15 see notes below about optimisation
'	  RS Pin used for RS
'     RD Pin used for RD
'     WR Pin used for WR
'     RST Pin used to generate Reset signal to display controller
'	  For optimised update, otherwise any valid pin
' 	  D0, Use 25
'     D1, Use 26
'     D2, Use 27
'     D3, Use 36
'     D4, Use 37
'     D5, Use 38
'     D6, Use 2
' 	  D7, Use 3
'     D8, Use 4
' 	  D9, Use 5
'	    D10, define the pin to use for DB10, any valid pin
'     D11, define the pin to use for DB11, any valid pin
'     D12, define the pin to use for DB13, any valid pin
'     D13, define the pin to use for DB13, any valid pin
'     D14: define the pin to use for DB14, any valid pin
'     D15: define the pin to use for DB15, any valid pin
'     RST: define the pin to use for REST, any valid pin
'
'initialise display:
'SUB S1289.init
'     no parameters
'
'
' clear screen: sub S1289.CLS(colour)
'    clear the screen to the colour specified
'    if the colour is omitted the screen is cleared to black
'
' set screen orientation: sub S1289.ROTATE(orientation)
'    valid values are PORTRAIT:0, RPORTRAIT:1, LANDSCAPE:2, RLANDSCAPE:3
'
' plot PIXEL: sub S1289.PIXEL(x, y, c) 
'    x, y coordinates of PIXEL, colour
'
' draw circle: sub S1289.CIRCLE(x, y, r, c, f)
'    x, y coordinates of centre, radius, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1289.BLACK() as the colour
'    to generate a circle in a single solid colour use the same colour for the outline and the fill
'
' draw triangle:  sub S1289.TRIANGLE(x0, y0, x1, y1, 	x2, y2, c, f) 
'    three coordinate pairs, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1289.BLACK() as the colour
'    to generate a triangle in a single solid colour use the same colour for the outline and the fill
'
' print text: sub S1289.TEXT(xx0, yy0, text$, font, size, forecolour, backcolour)
'    x, y coordinates of top left of first character, font number, text string, size, c,  foreground colour,  background colour
'    if the background colour is omitted black will be used
'
' print bitmap: sub S1289.BITMAP(x, y, bitmap, width, height, scale, forecolour, backcolour)
'    x, y coordinates of top left of the bitmap, bitmap, size, character orientation,  foreground colour,  background colour
'    the bitmap must be specifed as an array of integers
'    to output bitmap contained within a single integer use array size 1 and allocate the bimap to element 0
'
' draw line: sub S1289.LINE(x0, y0, x1, y1, colour) 
'    x, y of start line, x, y of end line, foreground colour,  background colour
'
' draw rectangle: sub S1289.BOX(x, y, w, h, c, f)
'    x, y coordinate of top left, width, height, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1289.BLACK() as the colour
'    to generate a rectangle in a single solid colour use the same colour for the outline and the fill
'
' draw rounded rectangle: sub S1289.RBOX(x, y, w, h, r, c, f)
'    x, y coordinates of top left, width, height, radius of corner, colour of outline, colour to fill
'    if colour to fill is omitted an outline only is drawn
'    to fill with black use the special function S1289.BLACK() as the colour
'    to generate a rounded rectangle in a single solid colour use the same colour for the outline and the fill
'
' screen horizontal resolution in default orientation:  S1289.MM.HRES.DEFAULT
' screen vertical resolution in default orientation:  S1289.MM.VRES.DEFAULT
' screen horizontal resolution in current orientation:  S1289.MM.HRES
' screen vertical resolution in current orientation:  S1289.MM.VRES
'
'
' Touch input routines
'
'initialise touch:
'SUB XPT.init(IRQ,CSxl%, xr%, yt%, yb%, xs%, ys%, n%)
'     calibration parameters for the touch screen, if not specified will automatically call XPT.CALIBRATE
'     number of samples at each touch to minimise noise, defaults to 5
'     CS: chip select pin for XPT touch controller, any valid pin
'     IRQ: interrupt pin for XPT touch controller, any valid pin
'     S1289_default_rotation: default screen orientation for this display type, in the case of the S1289 this is LANDSCAPE:2
'
'rotate screen keeping touch input correct :
'SUB XPT.ROTATE(N)
'    valid values for N are PORTRAIT:0, RPORTRAIT:1, LANDSCAPE:2, RLANDSCAPE:3
'    note if S1289.ROTATE is used rather than XPT.ROTATE when touch is active subsequent touch coordinates will be incorrect
'
'calibrate touch:
'SUB XPT.CALIBRATE
'    no parameters
'    sets and prints the following values
'    XPT.XSAMPLE0, XPT.YSAMPLE0, XPT.XLEFT0, XPT.XRIGHT0, XPT.YTOP0, XPT.YBOTTOM0
'    for any given display these will be fixed so can be set as constants for future use
'    this allows the calibrate routine to be removed for working systems
'    the related variables XPT.XSAMPLE, XPT.YSAMPLE, XPT.XLEFT, XPT.XRIGHT, XPT.YTOP, XPT.YBOTTOM are set for the default orientation
'    these are changed by XPT.ROTATE from the values stored in XPT.XSAMPLE0 etc.
'
'get touch coordinates:
'FUNCTION XPT.SCREENPOS(x, y, t)
'    returns true if a touch has occured within the timeout period
'    returns false if the timeout has occured
'    if true, x and y coordinates of position touched
'    timeout after which a false return will occur if no touch
'
' Display specific routines
'

'
sub S1289.init

	'Global variables
	'Command codes for S1289 CFunction
	DIM INTEGER S1289.pix=0, S1289.ffr=1, S1289.filltri=2, S1289.drawline=3, S1289.wr=5
	DIM INTEGER S1289.fpstring=7, S1289.fcirch=8, S1289.circh=9, S1289.Bits=10, S1289.scrolled=11, S1289.initialise=12

	'Orientation codes
	DIM INTEGER LANDSCAPE=2, RLANDSCAPE=3, PORTRAIT=0, RPORTRAIT=1
	
	DIM INTEGER S1289.MM.HRES.DEFAULT, S1289.MM.VRES.DEFAULT, S1289.MM.HRES, S1289.MM.VRES
  S1289.MM.HRES.DEFAULT=240: S1289.MM.VRES.DEFAULT=320

	S1289.VAR(S1289.OPTIMISE)=0
	IF S1289.VAR(S1289.DB0_I)=25 AND S1289.VAR(S1289.DB1_I)=26 AND S1289.VAR(S1289.DB2_I)=27 AND S1289.VAR(S1289.DB3_I)=36 THEN
		IF S1289.VAR(S1289.DB4_I)=37 AND S1289.VAR(S1289.DB5_I)=38 AND S1289.VAR(S1289.DB6_I)=2 AND S1289.VAR(S1289.DB7_I)=3 THEN
			IF S1289.VAR(S1289.DB8_I)=4 AND S1289.VAR(S1289.DB9_I)=5 THEN
				S1289.VAR(S1289.OPTIMISE)=1
			ENDIF
		ENDIF
	ENDIF
	
	S1289.VAR(S1289.wi)=S1289.MM.HRES.DEFAULT :  S1289.VAR(S1289.ht)=S1289.MM.VRES.DEFAULT
	S1289.VAR(S1289.PORTS)=S1289(S1289.initialise,S1289.VAR())
	
	'Reset the display controller
	PIN(S1289.VAR(S1289.RST_I))=1:  SETPIN S1289.VAR(S1289.RST_I),DOUT
	PAUSE 100
	PIN(S1289.VAR(S1289.RST_I))=0
	PAUSE 100
	PIN(S1289.VAR(S1289.RST_I))=1
	PAUSE 100
	S1289.cfg(&H00,&H0001): S1289.cfg(&H03,&HA8A4):	S1289.cfg(&H0C,&H0000)
	S1289.cfg(&H0D,&H080C):	S1289.cfg(&H0E,&H2B00):	S1289.cfg(&H1E,&H00B7)
	S1289.cfg(&H01,&H2B3F):	S1289.cfg(&H02,&H0600):	S1289.cfg(&H10,&H0000)
	S1289.cfg(&H11,&H6070):	S1289.cfg(&H05,&H0000):	S1289.cfg(&H06,&H0000)
	S1289.cfg(&H16,&HEF1C):	S1289.cfg(&H17,&H0003):	S1289.cfg(&H07,&H0233)
	S1289.cfg(&H0B,&H0000):	S1289.cfg(&H0F,&H0000):	S1289.cfg(&H41,&H0000)
	S1289.cfg(&H42,&H0000):	S1289.cfg(&H48,&H0000):	S1289.cfg(&H49,&H013F)
	S1289.cfg(&H4A,&H0000):	S1289.cfg(&H4B,&H0000):	S1289.cfg(&H30,&H0707)
	S1289.cfg(&H31,&H0204):	S1289.cfg(&H32,&H0204):	S1289.cfg(&H33,&H0502)
	S1289.cfg(&H34,&H0507):	S1289.cfg(&H35,&H0204):	S1289.cfg(&H36,&H0204)
	S1289.cfg(&H37,&H0502):	S1289.cfg(&H3A,&H0302):	S1289.cfg(&H3B,&H0302)
	S1289.cfg(&H23,&H0000):	S1289.cfg(&H24,&H0000):	S1289.cfg(&H25,&H8000)
  pause 50
  S1289.rotate(portrait)
end sub
'
' Set up the pin definitions
' Call S1289.PinDefs BEFORE calling anything else in S1289 
' RS, any valid pin
' RD, any valid pin, set to zero if tied high, scroll functionality not available if zero
' WR, any valid pin
' D0, Use 25 for optimised update and scroll functionality, otherwise any valid pin
' D1, Use 26 for optimised update and scroll functionality, otherwise any valid pin
' D2, Use 27 for optimised update and scroll functionality, otherwise any valid pin
' D3, Use 36 for optimised update and scroll functionality, otherwise any valid pin
' D4, Use 37 for optimised update and scroll functionality, otherwise any valid pin
' D5, Use 38 for optimised update and scroll functionality, otherwise any valid pin
' D6, Use 2 for optimised update and scroll functionality, otherwise any valid pin
' D7, Use 3 for optimised update and scroll functionality, otherwise any valid pin
' D8, Use 4 for optimised update and scroll functionality, otherwise any valid pin
' D9, Use 5 for optimised update and scroll functionality, set to zero if not using PortC pins for databus
'	D10, define the pin to use for DB10, any valid pin
' D11, define the pin to use for DB11, any valid pin
' D12, define the pin to use for DB13, any valid pin
' D13, define the pin to use for DB13, any valid pin
' D14: define the pin to use for DB14, any valid pin
' D15: define the pin to use for DB15, any valid pin
' RST: define the pin to use for REST, any valid pin
'
SUB S1289.PinDefs(D0%,D1%,D2%,D3%,D4%,D5%,D6%,D7%,D8%,D9%,D10%,D11%,D12%,D13%,D14%,D15%,RS%,RD%,WR%,RST%)

	'Holds Parameters for use in the Cfunction
	DIM INTEGER S1289.VAR(28) 
	
	'Subscripts in S1289.VAR for each parameter
	DIM INTEGER S1289.Default_Rotation=0 'default screen orientation for this display type, in this case PORTRAIT
	DIM INTEGER S1289.wi=0,S1289.ht=1,S1289.rot=2,S1289.nr=3,S1289.X=4,S1289.Y=5,S1289.seed=6  
	DIM INTEGER S1289.RS_I=7, S1289.WR_I=8
	
	'Subscripts for the 16 bit data bus 
  DIM integer S1289.RS_I=7, S1289.WR_I=8
  DIM integer S1289.DB0_I=9, S1289.DB1_I=10, S1289.DB2_I=11, S1289.DB3_I=12
  DIM integer S1289.DB4_I=13, S1289.DB5_I=14, S1289.DB6_I=15, S1289.DB7_I=16
  DIM INTEGER S1289.DB8_I=17, S1289.DB9_I=18, S1289.DB10_I=19, S1289.DB11_I=20
  DIM integer S1289.DB12_I=21, S1289.DB13_I=22, S1289.DB14_I=23, S1289.DB15_I=24
  DIM integer S1289.RD_I=25,S1289.PORTS=26,S1289.OPTIMISE=27,S1289.RST_I=28
 	pin(RST%)=1:  SETPIN RST%,dout	
	'the CFunction will execute the SETPIN commands itself
	S1289.VAR(S1289.RST_I)=RST% : 
	S1289.VAR(S1289.RS_I)=RS%     :  S1289.VAR(S1289.WR_I)=WR%    :  S1289.VAR(S1289.RD_I)=RD%
	S1289.VAR(S1289.DB0_I)=D0%    :  S1289.VAR(S1289.DB1_I)=D1%   :  S1289.VAR(S1289.DB2_I)=D2%
	S1289.VAR(S1289.DB3_I)=D3%    :  S1289.VAR(S1289.DB4_I)=D4%   :  S1289.VAR(S1289.DB5_I)=D5%
	S1289.VAR(S1289.DB6_I)=D6%    :  S1289.VAR(S1289.DB7_I)=D7%   :  S1289.VAR(S1289.DB8_I)=D8%
	S1289.VAR(S1289.DB9_I)=D9%    :  S1289.VAR(S1289.DB10_I)=D10% :  S1289.VAR(S1289.DB11_I)=D11%
	S1289.VAR(S1289.DB12_I)=D12%  :  S1289.VAR(S1289.DB13_I)=D13% :  S1289.VAR(S1289.DB14_I)=D14%
	S1289.VAR(S1289.DB15_I)=D15%
	
END SUB
'
sub S1289.cfg(r%, d%)
  local i%
  i%=S1289(S1289.wr, r%, d%, S1289.VAR())
end sub
'
'
' Generic drawing routines
'
'
sub S1289.CLS(c%)
  local integer j,i=S1289.VAR(S1289.rot)
  S1289.VAR(S1289.rot)=PORTRAIT
  j=S1289(S1289.ffr, c%, 0,0,S1289.VAR(S1289.wi),S1289.VAR(S1289.ht), S1289.VAR())
  S1289.VAR(S1289.rot)=i
end sub
'
sub S1289.CIRCLE(x%, y%, r%, c%, f%)
  local i%
  if(f%) then 
    i%=S1289(S1289.ffr, f%, x%, y%-r%, 1, 2*r%+1,  S1289.VAR())
    i%=S1289(S1289.fcircH, x%, y%, r%, 3, 0, f%, S1289.VAR())
  endif
  if(f%<>c%) then i%=S1289(S1289.circh, c%, x%, y%, r%, 31, S1289.VAR())
end sub
 '
function S1289.black()as integer
   S1289.black=&H8000000000000000
end function
'
sub S1289.PIXEL(x as integer, y as integer,  c as integer) 
  local i%=S1289(S1289.pix, c, x, y, S1289.VAR())
end sub
 '
sub S1289.TRIANGLE(x0 as integer, y0 as integer, x1 as integer, y1 as integer, x2 as integer, y2 as integer, c as integer, f as integer)
  local i%
  if(f) then i%=S1289(S1289.filltri, x0,y0,x1,y1,x2,y2,c,S1289.VAR())
  if(f<>c) then
     S1289.LINE(x0, y0, x1, y1, c)
     S1289.LINE(x1, y1, x2, y2, c)
     S1289.LINE(x2, y2, x0, y0, c)
  endif
end sub
'
sub S1289.TEXT(x%, y%, text$, font%, sz%, c%, f%) 
  local j%=font% and 1
  local integer i=S1289(S1289.fpstring,j%,x%,y%,sz%,c%,f%,S1289.VAR(),text$)
end sub
'
sub S1289.LINE(x0%, y0%, x1%, y1%, c%)
   local i%=S1289(S1289.drawline, c%, x0%, y0%, x1%, y1%,  S1289.VAR())
end sub
'
sub S1289.BOX(x as integer, y as integer, w as integer, h as integer, c as integer, f as integer)
  local integer a, i
  if(f) then i=S1289(S1289.ffr, f, x, y, w, h,  S1289.VAR()) 
  if(f<>c) then
     i=S1289(S1289.ffr, c, x, y, w, 1,  S1289.VAR()) 
     a=y+h-1
     i=S1289(S1289.ffr, c, x, a, w, 1,  S1289.VAR()) 
     i=S1289(S1289.ffr, c, x, y, 1, h,  S1289.VAR()) 
     a=x+w-1
     i=S1289(S1289.ffr, c, a, y, 1, h,  S1289.VAR()) 
  endif
end sub
'
sub S1289.BITMAP(x%, y%, array%(), w%, h%, sz%, c%, f%)
  local i%=S1289(S1289.Bits, x%,y%,w%,h%,sz%,c%,f%,array%(),S1289.VAR())  
end sub
'
sub S1289.RBOX(x as integer, y as integer, w as integer, h as integer, r as integer, c as integer, f as integer)
  local integer i
  if(f) then
	  i=S1289(S1289.ffr, f, x+r, y, w-2*r, h,  S1289.VAR()) 
	  i=S1289(S1289.fcircH,x+w-r-1, y+r, r, 1, h-2*r-1, f, S1289.VAR())
	  i=S1289(S1289.fcircH,x+r, y+r, r, 2, h-2*r-1, f, S1289.VAR())
  endif
  if(f<>c) then
	  i=S1289(S1289.ffr, c, x+r, y, w-2*r, 1,  S1289.VAR()) 
	  i=S1289(S1289.ffr, c, x+r, y+h-1, w-2*r, 1,  S1289.VAR()) 
	  i=S1289(S1289.ffr, c, x, y+r, 1, h-2*r,  S1289.VAR()) 
	  i=S1289(S1289.ffr, c, x+w-1, y+r, 1, h-2*r,  S1289.VAR()) 
	  i=S1289(S1289.circh, c, x+r, y+r, r, 1,S1289.VAR())
	  i=S1289(S1289.circh,  c, x+w-r-1, y+r, r, 2,S1289.VAR())
	  i=S1289(S1289.circh,  c, x+w-r-1, y+h-r-1, r, 4,S1289.VAR())
	  i=S1289(S1289.circh,  c, x+r, y+h-r-1, r, 8,S1289.VAR())
  endif
end sub
'
sub S1289.ROTATE(x%) 'sets screen rotation: 0=PORTRAIT, 1=RPORTRAIT, 2=LANDSCAPE, 3-RLANDSCAPE
    S1289.VAR(S1289.rot)=(x% and 3)
    local o% = &H6070
    select case x%
        case portrait
		     S1289.MM.VRES=S1289.MM.VRES.DEFAULT
		     S1289.MM.HRES=S1289.MM.HRES.DEFAULT
         case rportrait
		       S1289.MM.VRES=S1289.MM.VRES.DEFAULT
		       S1289.MM.HRES=S1289.MM.HRES.DEFAULT
           o% = &H6040
         case landscape
		       S1289.MM.VRES=S1289.MM.HRES.DEFAULT
		       S1289.MM.HRES=S1289.MM.VRES.DEFAULT
           o% = &H6068
         case rlandscape
		       S1289.MM.VRES=S1289.MM.HRES.DEFAULT
		       S1289.MM.HRES=S1289.MM.VRES.DEFAULT
           o% = &H6058
    end select
    S1289.cfg(&H11,o%) ' set address mode
end sub
'
' 
' 
' Touch specific routines
'
SUB XPT.init(IRQ%, CS%, xl%, xr%, yt%, yb%, xs%, ys%, n%)
  DIM INTEGER XPT.NSAMPLES=n% 'number of times to sample each touch point
  DIM INTEGER XPT.VAR(2)
  DIM INTEGER XPT.IRQ=0, XPT.CS=1
  XPT.VAR(XPT.IRQ)=IRQ%:XPT.VAR(XPT.CS)=CS%
  IF XPT.NSAMPLES=0 THEN XPT.NSAMPLES=5
  DIM INTEGER XPT.rbuf(400) 'buffer area to support scrolling
  'Global variables to hold calibration values, can be assigned here if calibration already run
  DIM INTEGER XPT.XLEFT0=xl%, XPT.XRIGHT0=xr%, XPT.YTOP0=yt%, XPT.YBOTTOM0=yb%, XPT.XSAMPLE0=xs%, XPT.YSAMPLE0=ys% 
  ' global variables to hold active values
  DIM INTEGER XPT.XLEFT, XPT.XRIGHT, XPT.YTOP, XPT.YBOTTOM 
  ' arrays to hold raw sample values
  DIM INTEGER XPT.Xvalues(XPT.NSAMPLES-1),XPT.Yvalues(XPT.NSAMPLES-1)
  'parameters to hold XPT commands, these may be reversed by the rotate routine
  DIM INTEGER XPT.XSAMPLE=0, XPT.YSAMPLE 
'
  PIN(XPT.VAR(XPT.CS))=1
  SETPIN XPT.VAR(XPT.IRQ),DIN
  SETPIN XPT.VAR(XPT.CS),DOUT
  SPI OPEN 1000000,3,8 
  IF xl%=0 OR xr%=0 OR yt%=0 OR yb%=0 OR xs%=0 OR ys%=0 THEN XPT.CALIBRATE
END SUB
'
SUB XPT.Rotate(r AS INTEGER)
  S1289.ROTATE(r)'set the screen rotation
  IF S1289.Default_Rotation\2=r\2 THEN
    XPT.XSAMPLE=XPT.XSAMPLE0
    XPT.YSAMPLE=XPT.YSAMPLE0
    IF r=S1289.Default_Rotation THEN
      XPT.XLEFT=XPT.XLEFT0: XPT.XRIGHT=XPT.XRIGHT0 : XPT.YTOP=XPT.YTOP0 : XPT.YBOTTOM=XPT.YBOTTOM0
    ELSE
      XPT.XLEFT=XPT.XRIGHT0: XPT.XRIGHT=XPT.XLEFT0 : XPT.YTOP=XPT.YBOTTOM0 : XPT.YBOTTOM=XPT.YTOP0
    ENDIF
  ELSE
    XPT.XSAMPLE=XPT.YSAMPLE0
    XPT.YSAMPLE=XPT.XSAMPLE0
    IF ((r MOD 2) = (S1289.Default_Rotation MOD 2)) THEN
      IF S1289.Default_Rotation=2 THEN
         XPT.XLEFT=XPT.YBOTTOM0
         XPT.XRIGHT=XPT.YTOP0
         XPT.YTOP=XPT.XLEFT0
         XPT.YBOTTOM=XPT.XRIGHT0
       ELSE
         XPT.XLEFT=XPT.YTOP0
         XPT.XRIGHT=XPT.YBOTTOM0
         XPT.YTOP=XPT.XRIGHT0
         XPT.YBOTTOM=XPT.XLEFT0
       ENDIF
    ELSE
      IF S1289.Default_Rotation=0 THEN
         XPT.XLEFT=XPT.YBOTTOM0
         XPT.XRIGHT=XPT.YTOP0
         XPT.YTOP=XPT.XLEFT0
         XPT.YBOTTOM=XPT.XRIGHT0
       ELSE
         XPT.XLEFT=XPT.YTOP0
         XPT.XRIGHT=XPT.YBOTTOM0
         XPT.YTOP=XPT.XRIGHT0
         XPT.YBOTTOM=XPT.XLEFT0
       ENDIF
     ENDIF
   ENDIF
END SUB
'
SUB XPT.Calibrate 
  LOCAL INTEGER x,y,rsz=(S1289.MM.VRES+S1289.MM.HRES)\100, roff=rsz\2, csz=rsz\3
  PRINT ""
  XPT.XSAMPLE0=&H90: XPT.YSAMPLE0=&HD0
  S1289.ROTATE(S1289.Default_Rotation)
  XPT.XSAMPLE=XPT.XSAMPLE0
  XPT.YSAMPLE=XPT.YSAMPLE0
  IF (rsz MOD 2) THEN rsz=rsz+1
  IF csz=0 THEN csz=1
  SWITCHXY:
  S1289.CLS(0) 
  S1289.BOX(0,S1289.MM.VRES\2-roff,rsz,rsz,rgb(white),rgb(white))
  S1289.TEXT(S1289.MM.HRES\2-csz*4*8,S1289.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(XPT.XLEFT0,y)
  IF XPT.XLEFT0>1000 AND XPT.XLEFT0<3000 THEN 'Switching orientation
    XPT.XSAMPLE0=&HD0:XPT.XSAMPLE=XPT.XSAMPLE0
    XPT.YSAMPLE0=&H90:XPT.YSAMPLE=XPT.YSAMPLE0
    GOTO SWITCHXY
  ENDIF
  PRINT "XPT.XSAMPLE0=",HEX$(XPT.XSAMPLE0)
  PRINT "XPT.YSAMPLE0=",HEX$(XPT.YSAMPLE0)
  PRINT "XPT.XLEFT0=",XPT.XLEFT0
  S1289.CLS(0) : PAUSE 1000
  S1289.BOX(S1289.MM.HRES-rsz,S1289.MM.VRES\2-roff,rsz,rsz,rgb(white),rgb(white))
  S1289.TEXT(S1289.MM.HRES\2-csz*4*8,S1289.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(XPT.XRIGHT0,y)
  PRINT "XPT.XRIGHT0=",XPT.XRIGHT0
  S1289.CLS(0) : PAUSE 1000
  S1289.BOX(S1289.MM.HRES\2-roff,0,rsz,rsz,rgb(white),rgb(white))
  S1289.TEXT(S1289.MM.HRES\2-csz*4*8,S1289.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(x,XPT.YTOP0)
  PRINT "XPT.YTOP0=",XPT.YTOP0
  S1289.CLS(0) : PAUSE 1000
  S1289.BOX(S1289.MM.HRES\2-roff,S1289.MM.VRES-rsz,rsz,rsz,rgb(white),rgb(white))
  S1289.TEXT(S1289.MM.HRES\2-csz*4*8,S1289.MM.VRES\2-csz*4,"Touch box",0,csz,rgb(green),0)
  XPT.GetTouch(x,XPT.YBOTTOM0)
  PRINT "XPT.YBOTTOM0=",XPT.YBOTTOM0
  S1289.CLS(0) : PAUSE 1000
  XPT.XLEFT=XPT.XLEFT0: XPT.XRIGHT=XPT.XRIGHT0 : XPT.YTOP=XPT.YTOP0 : XPT.YBOTTOM=XPT.YBOTTOM0 ' set the parameters for the default orientation
  'for a particular application these values can be saved and set as CONSTants
END SUB
    '
FUNCTION XPT.Screenpos(x AS INTEGER, y AS INTEGER, t%) AS INTEGER
    LOCAL INTEGER j
    IF XPT.GetXY(XPT.Xvalues(),XPT.Yvalues(),t%,XPT.NSAMPLES)<>0 THEN
      '      j=shellsortARM(XPT.Xvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      j=shellsort(XPT.Xvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      '      j=shellsortARM(XPT.Yvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      j=shellsort(XPT.Yvalues(),XPT.NSAMPLES) 'uncomment to use sort as required
      x=XPT.Xvalues(XPT.NSAMPLES\2) 'get the median value
      y=XPT.Yvalues(XPT.NSAMPLES\2) 'get the median value
      IF XPT.XLEFT>XPT.XRIGHT THEN 'x coordinates reversed
        IF x<XPT.XRIGHT THEN x=XPT.XRIGHT
        IF x>XPT.XLEFT THEN x=XPT.XLEFT
        x=S1289.MM.HRES-((x-XPT.XRIGHT)*S1289.MM.HRES/(XPT.XLEFT-XPT.XRIGHT))
      ELSE 'x coordinates normal
        IF x<XPT.XLEFT THEN x=XPT.XLEFT
        IF x>XPT.XRIGHT THEN x=XPT.XRIGHT
        x=(x-XPT.XLEFT)*S1289.MM.HRES/(XPT.XRIGHT-XPT.XLEFT)
      ENDIF
      IF XPT.YTOP>XPT.YBOTTOM THEN ' y coordinates reversed
        IF y>XPT.YTOP THEN y=XPT.YTOP
        IF y<XPT.YBOTTOM THEN y=XPT.YBOTTOM
        y=S1289.MM.VRES-((y-XPT.YBOTTOM)*S1289.MM.VRES/(XPT.YTOP-XPT.YBOTTOM))
      ELSE 'y coordinates normal
        IF y<XPT.YTOP THEN y=XPT.YTOP
        IF y>XPT.YBOTTOM THEN y=XPT.YBOTTOM
        y=(y-XPT.YTOP)*S1289.MM.VRES/(XPT.YBOTTOM-XPT.YTOP)
      ENDIF
      XPT.Screenpos=1
    ELSE
      XPT.Screenpos=0
    ENDIF
END FUNCTION
    
SUB XPT.GetTouch(x AS INTEGER, y AS INTEGER)
    LOCAL INTEGER j, n=51,XPT.Xvalues(n-1),XPT.Yvalues(n-1)
loophere:
    x=-500
    IF XPT.GetXY(XPT.Xvalues(),XPT.Yvalues(),1000,n)<>0 THEN
      j=shellsort(XPT.Xvalues(),n) 'change to use sort as required
      j=shellsort(XPT.Yvalues(),n) 'change to use sort as required
      x=XPT.Xvalues(n\2) 'get the median value
      y=XPT.Yvalues(n\2) 'get the median value
    ENDIF
    IF x=-500 THEN GOTO loophere
END SUB
    '
FUNCTION XPT.GetData(command AS INTEGER) AS INTEGER 'get data from the touch controller
    LOCAL INTEGER i,j,k
    PIN(XPT.VAR(XPT.CS))=0
    i=SPI(command)
    j=SPI(0)
    k=SPI(0)
    PIN(XPT.VAR(XPT.CS))=1
    XPT.GetData= (j*256+k)>>4
END FUNCTION
    '
FUNCTION XPT.GetXY(x() AS INTEGER, y() AS INTEGER, timeout AS INTEGER, samples AS INTEGER) AS INTEGER
    LOCAL INTEGER i
    TIMER=0
    XPT.GetXY=1
    FOR i=0 TO samples-1
      DO WHILE TIMER<timeout AND PIN(XPT.VAR(XPT.IRQ))=1:LOOP
      IF TIMER>=timeout THEN
        XPT.GetXY=0
      ELSE
        y(i)=XPT.GetData(XPT.YSAMPLE)
        x(i)=XPT.GetData(XPT.XSAMPLE)
      ENDIF
    NEXT i
END FUNCTION
'
CFunction S1289
	000004F6
	'psend
	8ca800d0 8ca60030 00863026 aca40030 000417c3 10c0001a aca20034 8ca200d8 
	8ca300dc 00431025 50400006 24040001 3c02bf88 ac446230 24040400 10000002 
	2402001d 24020009 00021880 00681821 00862824 50a00005 24420002 8c650000 
	8c670004 aca70000 24420002 28450029 10a00003 24630008 1000fff5 00042040 
	8d020010 8d030014 ac430000 8d02000c 8d030014 ac430000 03e00008 00000000 

	'wr
	27bdffe0 afbf001c afb20018 afb10014 afb00010 00a09021 00c08821 8cd000d0 
	8e020004 8e030008 ac430000 00c02821 0411FFCB 00000000 8e020000 8e030008 
	ac430000 02402021 02202821 0411FFC4 00000000 8fbf001c 8fb20018 8fb10014 
	8fb00010 03e00008 27bd0020 

	'initdisplay
	27bdffc8 afbf0034 afb70030 afb6002c afb50028 afb40024 afb30020 afb2001c 
	afb10018 afb00014 0080b021 3c109d00 8e02003c 0040f809 24040100 0040b821 
	8e020024 8ec40038 0040f809 24050006 aee20000 8e020024 8ec40038 0040f809 
	24050005 aee20004 8e020028 0040f809 8ec40038 24110001 00511004 aee20008 
	8e020024 8ec40040 0040f809 24050006 aee2000c 8e020024 8ec40040 0040f809 
	24050005 aee20010 8e020028 0040f809 8ec40040 00511004 aee20014 02c08821 
	26f00024 26f400a4 02c09021 3c139d00 24150001 8e620024 8e440048 0040f809 
	24050007 ae020000 8e620028 0040f809 8e440048 00551004 ae020004 26100008 
	1614fff4 26520008 24120007 3c109d00 24130019 8e020010 8e240038 24050008 
	0040f809 00003021 8e02001c 8e240038 0040f809 24050005 26520001 1653fff5 
	26310008 3c109d00 8e02001c 8ec40038 0040f809 24050006 8e02001c 8ec40040 
	0040f809 24050006 00001021 00001821 aec20030 aec30034 8ec400c8 8ec200cc 
	00821025 1040001b 02e01021 8e020024 0040f809 24050006 aee200a4 8e020024 
	8ec400c8 0040f809 24050005 aee200a8 8e020028 0040f809 8ec400c8 24030001 
	00431004 aee200ac 8e020010 8ec400c8 24050008 0040f809 00003021 8e02001c 
	8ec400c8 0040f809 24050006 02e01021 00001821 8fbf0034 8fb70030 8fb6002c 
	8fb50028 8fb40024 8fb30020 8fb2001c 8fb10018 8fb00014 03e00008 27bd0038 

	'defineregion
	27bdffd0 afbf002c afb50028 afb40024 afb30020 afb2001c afb10018 afb00014 
	8fb00040 2488ffff 01064021 24a6ffff 00c73021 8e090010 8e0a0000 8e070008 
	15200008 8e1100d0 00801821 01001021 00a09821 00c09021 00a0a821 1000001a 
	0080a021 240b0001 152b000c 240b0002 00084027 010a1821 00041027 004a1021 
	00063027 00c79821 00059027 02479021 0240a821 10000016 0040a021 552b000b 
	240a0003 01463023 24c3ffff 00051027 004a1021 00809821 01009021 0080a821 
	1000000b 0040a021 240a0003 152a0008 00084027 00a01821 00c01021 01079821 
	00049027 02479021 0240a821 00a0a021 00021200 24040044 00432821 02003021 
	0411FF1C 00000000 24040045 02602821 02003021 0411FF17 00000000 24040046 
	02402821 02003021 0411FF12 00000000 2404004e 02802821 02003021 0411FF0D 
	00000000 2404004f 02a02821 02003021 0411FF08 00000000 8e220004 8e230008 
	ac430000 24040022 02002821 0411FED9 00000000 8e220000 8e230008 ac430000 
	8fbf002c 8fb50028 8fb40024 8fb30020 8fb2001c 8fb10018 8fb00014 03e00008 
	27bd0030 

	'localfastfillrect
	27bdffd0 afbf002c afb30028 afb20024 afb10020 afb0001c 00808021 8fa20040 
	8fb10044 70479802 afb10010 00a02021 00c02821 00e03021 00403821 0411FF87 
	00000000 7e0220c0 00109203 3252f800 00529025 00108143 321007e0 1260000a 
	02509025 2670ffff 2413ffff 02402021 02202821 0411FEAE 00000000 2610ffff 
	1613fffb 02402021 24020001 00001821 8fbf002c 8fb30028 8fb20024 8fb10020 
	8fb0001c 03e00008 27bd0030 

	'DBitmap
	27bdff88 afbf0074 afbe0070 afb7006c afb60068 afb50064 afb40060 afb3005c 
	afb20058 afb10054 afb00050 afa40078 00a08021 afa60080 afa70084 8fb20088 
	8fa3008c 8fa50098 8ca40010 2c840002 10800005 8fa20090 8cb30000 8ca40008 
	10000004 afa40034 8cb30008 8ca40000 afa40034 7c6520c0 00032203 3084f800 
	00a42025 00031943 306307e0 00831825 afa30028 7c4420c0 00021a03 3063f800 
	00831825 00021143 304207e0 00621025 afa2002c 8fa50098 afa50010 8fa40078 
	02002821 8fa20080 72423002 8fa20084 72423802 0411FF36 00000000 8fa30084 
	18600068 8fbf0074 afb20020 afb00038 8fa40080 afa40044 00042823 afa50048 
	00640018 00001012 afa20040 afa0001c 10000058 afa0003c 06000015 0213102b 
	50400014 26310001 06a20012 26310001 52e00010 26310001 93c20000 8fa30018 
	00431024 10400007 8fa4002c 8fa40028 8fa50098 0411FE4B 00000000 10000005 
	26310001 8fa50098 0411FE46 00000000 26310001 1632ffe8 26100001 26940001 
	8fa40020 8fa50080 12850016 02c4b021 8fa3001c 02831021 24430007 285e0000 
	007e100b 0002f0c3 8fa40094 009ef021 8fa50024 00b41823 000317c3 00021742 
	00621821 30630007 00621023 24030001 00431004 afa20018 02c08021 1000ffce 
	00008821 8fa40030 24840001 afa40030 14920007 26b50001 1000000c 8fa4003c 
	afa00030 8fa50040 24a5ffff afa50024 8fa20080 1840fff3 8fb60078 0000a021 
	8fa30034 1000ffda 02a3b82b 24840001 afa4003c 8fa50038 8fa20020 00a22821 
	afa50038 8fa3001c 8fa40044 00641821 afa3001c 8fa50040 8fa20048 00a22821 
	afa50040 8fa3003c 8fa40084 10640005 8fbf0074 1e40ffe2 8fb50038 1000ffeb 
	8fa4003c 8fbe0070 8fb7006c 8fb60068 8fb50064 8fb40060 8fb3005c 8fb20058 
	8fb10054 8fb00050 03e00008 27bd0078 

	'swap
	8c820000 8ca30000 ac830000 03e00008 aca20000 

	'drawline
	27bdffc0 afbf003c afb70038 afb60034 afb50030 afb4002c afb30028 afb20024 
	afb10020 afb0001c 0080a021 afa50044 afa60048 afa7004c 8fa20050 14c20016 
	8fb50054 00e5102a 10400006 8fa20044 27a40044 27a5004c 0411FFE4 00000000 
	8fa20044 8fa7004c 24e70001 24030001 afa30010 afb50014 02802021 00402821 
	8fa60048 00e23823 0411FF01 00000000 10000049 8fbf003c 8fb60044 8fa3004c 
	16c30015 00769823 0046102a 10400006 8fa60048 27a40048 27a50050 0411FFCB 
	00000000 8fa60048 8fa20050 24420001 00461023 afa20010 afb50014 02802021 
	8fa50044 24070001 0411FEE9 00000000 10000031 8fbf003c 001327c3 00939826 
	02649823 02c3b02a 24040001 2405ffff 00b6200a 0080b021 00469023 00121fc3 
	00729026 00729023 00c2b82a 24030001 2404ffff 0097180a 0060b821 02538021 
	24110001 afb10010 afb50014 02802021 8fa50044 8fa60048 02203821 0411FECC 
	00000000 00101040 0052182a 54600008 0262102a 8fa30044 8fa4004c 1064000d 
	02128021 02c31821 afa30044 0262102a 5440ffed afb10010 8fa20048 8fa30050 
	10430004 02138021 02e21021 1000ffe5 afa20048 8fbf003c 8fb70038 8fb60034 
	8fb50030 8fb4002c 8fb30028 8fb20024 8fb10020 8fb0001c 03e00008 27bd0040 

	'drawchar
	27bdffd0 afbf002c 8faa0040 8fa90044 8fa20048 8fab004c 7c042420 3c039d00 
	8c630054 00052880 00a31821 8c650000 90a80001 90ac0002 008c682a 15a00019 
	90a30000 90ad0003 01ac6821 008d682a 51a00015 71024002 afa20010 afa60014 
	afa70018 71031002 000210c3 008c6023 704c2002 24840004 00a42821 afa5001c 
	afab0020 01402021 01202821 00603021 01003821 0411FEB1 00000000 1000000b 
	8fbf002c 71024002 afa80010 afab0014 00e02021 01402821 01203021 70623802 
	0411FE7B 00000000 8fbf002c 03e00008 27bd0030 

	'testchar
	8fa30010 8c680010 8c690000 70e61002 00442021 2d020002 10400009 8c630008 
	04800012 00001021 04a00010 0124302a 14c0000e 0065302a 14c0000c 00000000 
	2508fffe 2d080002 11000008 24020001 04800006 00001021 04a00004 0064182a 
	14600002 0125282a 38a20001 03e00008 00000000 

	'pstring
	27bdffb8 afbf0044 afbe0040 afb7003c afb60038 afb50034 afb40030 afb3002c 
	afb20028 afb10024 afb00020 0080b821 00a08021 00c09821 00e0a021 8fb50064 
	3c029d00 8c420054 00041880 00621021 8c420000 90560000 92a20000 1840001d 
	8fb20060 72c7f002 24110001 afb20010 02002021 02602821 02803021 02c03821 
	0411FFC2 00000000 10400019 02b11021 80440000 afb00010 afb30014 afb40018 
	afb2001c 02e02821 8fa60058 8fa7005c 0411FF81 00000000 00404021 00604821 
	26310001 92a20000 0051102a 1040ffe7 021e8021 ae500020 001087c3 ae500024 
	ae530028 00139fc3 10000003 ae53002c 00004021 00004821 01001021 01201821 
	8fbf0044 8fbe0040 8fb7003c 8fb60038 8fb50034 8fb40030 8fb3002c 8fb20028 
	8fb10024 8fb00020 03e00008 27bd0048 

	'dcirch
	27bdffb0 afbf004c afbe0048 afb70044 afb60040 afb5003c afb40038 afb30034 
	afb20030 afb1002c afb00028 0080b021 00a0a821 00c0a021 00e08821 8fa30060 
	30620010 10400020 8fb70064 24100001 afb00010 afb70014 00e63021 24070001 
	0411FDF5 00000000 afb00010 afb70014 02c02021 02a02821 02913023 24070001 
	0411FDED 00000000 afb00010 afb70014 02c02021 02352821 02803021 24070001 
	0411FDE5 00000000 afb00010 afb70014 02c02021 02b12823 02803021 24070001 
	0411FDDD 00000000 1a200069 24130001 02719823 0011f023 001ef040 00008021 
	8fa20060 30420004 afa2001c 24120001 8fa30060 30630002 afa30020 8fa20060 
	30420008 afa20024 8fa30060 30630001 afa30018 06600004 8fa2001c 2631ffff 
	27de0002 027e9821 10400011 26100001 afb20010 afb70014 02c02021 02152821 
	02343021 02403821 0411FDBB 00000000 afb20010 afb70014 02c02021 02352821 
	02143021 02403821 0411FDB3 00000000 8fa30020 10600012 8fa20024 afb20010 
	afb70014 02c02021 02152821 02913023 02403821 0411FDA8 00000000 afb20010 
	afb70014 02c02021 02352821 02903023 02403821 0411FDA0 00000000 8fa20024 
	10400012 8fa30018 afb20010 afb70014 02c02021 02b12823 02143021 02403821 
	0411FD95 00000000 afb20010 afb70014 02c02021 02b02823 02343021 02403821 
	0411FD8D 00000000 8fa30018 10600012 0211102a afb20010 afb70014 02c02021 
	02b12823 02903023 02403821 0411FD82 00000000 afb20010 afb70014 02c02021 
	02b02823 02913023 02403821 0411FD7A 00000000 0211102a 10400006 8fbf004c 
	00101040 24420001 1000ffaa 02629821 8fbf004c 8fbe0048 8fb70044 8fb60040 
	8fb5003c 8fb40038 8fb30034 8fb20030 8fb1002c 8fb00028 03e00008 27bd0050 

	'fcirch
	27bdffb0 afbf004c afbe0048 afb70044 afb60040 afb5003c afb40038 afb30034 
	afb20030 afb1002c afb00028 0080b821 00a0b021 8fb50068 18c00048 8fbe0070 
	24100001 02068023 0006a023 0014a040 00c08821 24130003 00009021 30e20001 
	afa20018 8fa30060 24630001 afa30020 30e70002 afa7001c 06000004 8fa20018 
	2631ffff 26940002 02148021 10400016 26520001 00111040 8fa30020 00621021 
	afa20010 afbe0014 02a02021 02572821 02d13023 24070001 0411FD37 00000000 
	8fa30060 02631021 afa20010 afbe0014 02a02021 02372821 02d23023 24070001 
	0411FD2D 00000000 8fa2001c 10400017 0251102a 00111040 8fa30020 00621021 
	afa20010 afbe0014 02a02021 02f22823 02d13023 24070001 0411FD1F 00000000 
	8fa30060 02631021 afa20010 afbe0014 02a02021 02f12823 02d23023 24070001 
	0411FD15 00000000 0251102a 10400003 02138021 1000ffc8 26730002 8fbf004c 
	8fbe0048 8fb70044 8fb60040 8fb5003c 8fb40038 8fb30034 8fb20030 8fb1002c 
	8fb00028 03e00008 27bd0050 

	'ftri
	27bdffc8 afbf0034 afb40030 afb3002c afb20028 afb10024 afb00020 afa40038 
	afa5003c afa60040 afa70044 8fb20050 00e5282a 10a00009 8fb10054 27a4003c 
	27a50044 0411FDC8 00000000 27a40038 27a50040 0411FDC4 00000000 8fa2004c 
	8fa30044 0043102a 1040000a 8fa20044 27a4004c 27a50044 0411FDBB 00000000 
	27a40048 27a50040 0411FDB7 00000000 8fa20044 8fa3003c 0043102a 1040000a 
	8fb0003c 27a4003c 27a50044 0411FDAE 00000000 27a40038 27a50040 0411FDAA 
	00000000 8fb0003c 8fa2004c 16020023 8fb30044 8fa20038 afa20018 afa2001c 
	8fa30040 0062202a 10800003 0043102a 10000003 afa30018 54400001 afa3001c 
	8fa20048 8fa30018 0043182a 10600003 8fa3001c 10000004 afa20018 0062182a 
	54600001 afa2001c 8fa50018 8fa2001c 24420001 00451023 afa20010 afb10014 
	02402021 8fa6003c 24070001 0411FCAF 00000000 10000068 8fbf0034 00531026 
	0002102b 02629823 0270102a 14400030 8fa2004c 24140001 8fa20038 8fa3003c 
	02032823 8fa60040 00c23023 70a63002 8fa40044 00832023 00c4001a 008001f4 
	00002012 00822021 afa40018 8fa60048 00c23023 70a62802 8fa6004c 00c31823 
	00a3001a 006001f4 00001812 00621021 afa2001c 0044102a 10400006 8fa20018 
	27a40018 27a5001c 0411FD5F 00000000 8fa20018 8fa7001c 24e70001 afb40010 
	afb10014 02402021 00402821 02003021 00e23823 0411FC7D 00000000 26100001 
	0270102a 1040ffd5 8fa20038 8fa2004c 0050182a 1460002f 24130001 8fa60040 
	8fa40038 8fa50048 8fa80044 02083823 00a61823 70e33802 00484023 00e8001a 
	010001f4 00001812 00661821 afa30018 8fa7003c 02073023 00a42823 70c53002 
	00471023 00c2001a 004001f4 00001012 00442021 afa4001c 0083202a 10800006 
	8fa20018 27a40018 27a5001c 0411FD2E 00000000 8fa20018 8fa7001c 24e70001 
	afb30010 afb10014 02402021 00402821 02003021 00e23823 0411FC4C 00000000 
	26100001 8fa2004c 0050182a 1060ffd4 8fa60040 8fbf0034 8fb40030 8fb3002c 
	8fb20028 8fb10024 8fb00020 03e00008 27bd0038 

	'main
	27bdffd0 afbf002c afb00028 00808021 8fa80040 8faa0044 8fa90048 8fab004c 
	8c820000 1440000d 8fac0050 8ca40000 8cc50000 8ce60000 24020001 afa20010 
	afa80014 24070001 0411FC2B 00000000 00402021 10000086 00022fc3 24030001 
	1443000d 24030002 8ca40000 8cc50000 8ce60000 8d070000 8d420000 afa20010 
	afa90014 0411FC1C 00000000 00402021 10000077 00022fc3 14430011 24030003 
	8ca40000 8cc50000 8ce60000 8d070000 8d420000 afa20010 8d220000 afa20014 
	8d620000 afa20018 afac001c 0411FF07 00000000 8e040000 10000065 00042fc3 
	1443000d 24030005 8ca40000 8cc50000 8ce60000 8d070000 8d420000 afa20010 
	afa90014 0411FCD8 00000000 8e040000 10000057 00042fc3 14430009 24030007 
	8ca40000 8cc50000 00e03021 0411FAE6 00000000 8e040000 1000004d 00042fc3 
	14430010 24030008 8ca40000 8cc50000 8ce60000 8d070000 8d420000 afa20010 
	8d220000 afa20014 afab0018 afac001c 0411FD87 00000000 00402021 1000003c 
	00022fc3 14430011 24030009 8ca40000 8cc50000 8ce60000 8d070000 8d420000 
	afa20010 8d220000 afa20018 000217c3 afa2001c afab0020 0411FE69 00000000 
	8e040000 1000002a 00042fc3 1443000d 2403000a 8ca40000 8cc50000 8ce60000 
	8d070000 8d420000 afa20010 afa90014 0411FDB3 00000000 8e040000 1000001c 
	00042fc3 14430013 2403000c 8ca40000 8cc50000 8ce60000 8d070000 8d420000 
	afa20010 8d220000 afa20014 8d620000 afa20018 afac001c 8fa20054 afa20020 
	0411FBD8 00000000 00002021 10000008 00002821 14430009 8fbf002c 00a02021 
	0411FAB4 00000000 00402021 00022fc3 00801021 00a01821 8fbf002c 8fb00028 
	03e00008 27bd0030 
End CFunction   'MIPS32 M4K
'
CFunction shellsort
  00000000
  8ca20000 27bdfff8 afb10004 afb00000 00401821 00032fc2 00a31821 00031843
  5060003d 00001021 0062282a 10a0fffa 00032fc2 00036823 000370c0 000d60c0
  008e7021 0080c021 008c8021 00607821 0000c821 01e3282a 01c03021 8dd10000
  14a0001e 8dcb0004 8f070004 03202821 0167402a 11000034 8f090000 01e04021
  000840c0 00884021 00a3502a 02003021 ad090000 1540000f ad070004 8cc70004
  8cc90000 0167402a 00ad5021 1100001f 00cc3021 00a04021 01402821 000840c0
  00884021 00a3502a ad090000 1140fff3 ad070004 000528c0 00853021 25ef0001
  01e2282a acd10000 accb0004 27390001 25ce0008 27180008 14a0ffd6 26100008
  00032fc2 00a31821 00031843 1460ffc7 0062282a 00001021 00001821 8fb10004
  8fb00000 03e00008 27bd0008 14ebffe9 0229402b 5100ffe8 000528c0 00a04021
  1000ffde 01402821 54ebffe5 25ef0001 0229402b 5500ffca 01e04021 1000ffe0
  25ef0001
End CFunction
'
'----------------------------------------------------------------------------
	'Cut here and throw away below when using the Library in your own program
testsuite:
'      
'
' Define the pins to use for all connections to the display
'
	PRINT "start Init"
	
	'Tell the library which pins we're using for the display
	'Connect CS to GND
	'SUB S1289.PinDefs(D0,D1,D2,D3,D4,D5,D6,D7,D8,D9,D10,D11,D12,D13,D14,D15,RS,RD,WR,RST)
	S1289.PinDefs(25,26,27,36,37,38,2,3,4,5,30,31,21,22,11,24,12,13,23,32)
	S1289.init 'initialise the S1289 display
	
	'CS  = pin 42  ' chip select pin for XPT touch controller, any valid pin
	'IRQ = pin 43  ' interrupt pin for XPT touch controller, any valid pin
	' Connect T_DO to SPIIN (pin 14 on 28-pin, pin 41 on 44-pin)
	' Connect T_CLK to SPICLK (pin 25 on 28-pin,pin 14 on 44-pin)
	' Connect T_DIN to SPIOUT (pin 3 on 28-pin, pin 20 on 44-pin)

	'SUB XPT.init(IRQ,CSxl%, xr%, yt%, yb%, xs%, ys%, n%)
	'initialise the touch system, just supply the IRQ & CS pin numbers 
	'no paramaters so calibration will be instigated
	XPT.Init(43,42) 
	
	PRINT "Start main"

	DO
	   S1289TestSuite
	LOOP

END
sub S1289TestSuite
  local integer k,x,y 'main program variables
  local integer i, j, x0, y0, x1, y1, sz, bitmap(1)=(&HFFC1A191898583FF,0)
  local float tn
  S1289.ROTATE(LANDSCAPE)
  S1289.CLS()
  S1289.BOX(0, 0,S1289.MM.HRES,S1289.MM.VRES,rgb(red))
  for i=10 to 0 step -1
    S1289.TEXT(S1289.MM.HRES/2-48, 70, str$(i,2) ,1 ,2, rgb(green))
    pause 1000
  next i
  S1289.TEXT(S1289.MM.HRES/2-48, 70, "  " ,1 ,2, rgb(green))
  for i=0 to 3
     S1289.ROTATE(i)
     S1289.BITMAP(10,10,bitmap(),8,8,3,rgb(blue))
  next i  
  S1289.ROTATE(LANDSCAPE)
  S1289.TEXT(S1289.MM.HRES/2-112, 60, "Welcome to the", 0,2, rgb(green))
  S1289.TEXT(S1289.MM.HRES/2-84, 100,  "Micromite MKII 4.7b8", 0,1, rgb(red))
  S1289.TEXT(S1289.MM.HRES/2-108, 130, "in colour", 0,3, rgb(yellow))
  S1289.TEXT(S1289.MM.HRES/2-84, 200,  "16-bit parallel S1289", 0,1, rgb(cyan))
  pause 4000
' demo fast line draw
  S1289.CLS()
  S1289.ROTATE(RLANDSCAPE)
  x0=0:y0=0:y1=S1289.MM.VRES-1
  for i=0 to 44 step 2
    tn=tan(rad(i))
    x1=S1289.MM.HRES*tn
    S1289.LINE(x0, y0, x1, y1, rgb(red))
  next i
  x1=S1289.MM.HRES-1
  for i=0 to 44 step 2
    tn=tan(rad(i))
    y1=S1289.MM.VRES*tn
    S1289.LINE(x0, y0, x1, y1, rgb(red))
  next i
  S1289.BOX(0, 0, S1289.MM.HRES, S1289.MM.VRES, rgb(blue))
  pause 2000

  S1289.CLS()
'demo fast text write
  S1289.ROTATE(LANDSCAPE)

  for i=0 to S1289.MM.VRES-13 step 13
    S1289.TEXT(0, i, "Hello world:Bye world:Hello world:Bye world:Hello world:Bye world:Hello world:Bye world", 0,1, rgb(white))
  next i
  pause 2000
  S1289.ROTATE(RLANDSCAPE)
  S1289.CLS()
  for i=0 to S1289.MM.VRES-26 step 26
    S1289.TEXT(0, i, "Micromite MK2:Invert:Micromite MK2:Invert", 0,2, rgb(red))
  next i
  pause 2000
  S1289.ROTATE(RPORTRAIT)
  S1289.CLS()
  for i=0 to S1289.MM.VRES-39 step 39
    S1289.TEXT(0,i, "Rotate right:Rotate right:Rotate right", 0,3, rgb(yellow))
  next i
  pause 2000
  S1289.ROTATE(PORTRAIT)
  S1289.CLS()
  for i=0 to S1289.MM.VRES-52 step 52
    S1289.TEXT(0,i, "Rotate Left:Rotate Left:Rotate Left", 0,3, rgb(blue))
  next i
  pause 2000
  S1289.CLS()
  ' test drawing rectangles
  for i=0 to 3
     S1289.ROTATE(i)
     S1289.BOX(5, 5, 80, 90, rgb(white), rgb(blue))
     S1289.BOX(2, 2, 86, 96, rgb(red))
  next i
  pause 2000
' test drawing rounded rectangles
  S1289.CLS()
  for i=0 to 3
     S1289.ROTATE(i)
     S1289.RBOX(5, 5, 80, 45, 10, rgb(blue), rgb(blue))
     S1289.RBOX(2, 2, 86, 51, 12, rgb(red))
  next i
  pause 2000
  S1289.CLS()
' test drawing circles
  for i=0 to 3
     S1289.ROTATE(i)
     S1289.CIRCLE(40, 40, 35, rgb(blue), rgb(blue))
     S1289.CIRCLE(40, 40, 39, rgb(red))
  next i
  pause 2000
' test drawing triangles and line drawing
  S1289.CLS()
  for i=0 to 3
     S1289.ROTATE(i)
     S1289.TRIANGLE(4, 4, 100, 6, 22, 104, rgb(blue), rgb(blue))
     S1289.TRIANGLE(1, 2, 104, 4, 22, 107, rgb(red))
  next i
  pause 2000
  for j=0 to 3
     XPT.rotate(j)
     S1289.CLS(0)
     S1289.TEXT(0,0,"Top left",0,2,rgb(green))
     k=0
main:
     if XPT.screenpos(x,y,1000) then
       if x>3 and x<S1289.MM.HRES-3 and y>3 and y<S1289.MM.VRES-3 then S1289.CIRCLE(x,y,3,rgb(white),rgb(white))
     else 
        k=k+1
     endif
     if k<10 goto main
  next j
end SUB
